<?php
/*
 * Character references:
 *  Demonstrate the use of name-based and numerical character references.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 * Required data: font file
 */

/* This is where the data files are. Adjust as necessary. */
$searchpath = dirname(__FILE__,3)."/input";
$outfile = "";
$title = "Character References";

$x=20; 
$y=220; 
$width=300; 
$height=300;

try {
    $p = new PDFlib();

    $p->set_option("searchpath={" . $searchpath . "}");

    /* This means we must check return values of load_font() etc. */
    $p->set_option("errorpolicy=return");
    $p->set_option("stringformat=utf8");

    if ($p->begin_document($outfile, "") == 0)
    throw new Exception("Error: " . $p->get_errmsg());

    $p->set_info("Creator", "PDFlib Cookbook");
    $p->set_info("Title", $title);

    $font = $p->load_font("NotoSerif-Regular", "unicode", "");
    if ($font == 0)
        throw new Exception("Error: " . $p->get_errmsg());

    /* Start page */
    $p->begin_page_ext($width, $height, "");

    $p->fit_textline("Input", $x, $y, "font=" . $font . " fontsize=16");
    $p->fit_textline("Output", $x+160, $y, "font=" . $font .
        " fontsize=20");

   /* Output the Euro glyph via a character reference */
    $p->fit_textline("500 &euro;", $x, $y-=24, "font=" . $font .
        " fontsize=16 charref=false");
    $p->fit_textline("500 &euro;", $x+160, $y, "font=" . $font .
        " fontsize=16 charref=true");

    /*  Output the Euro glyph via a character reference */
    $p->fit_textline("500 &#x20AC;", $x, $y-=24, "font=" . $font .
        " fontsize=16 charref=false");
    $p->fit_textline("500 &#x20AC;", $x+160, $y, "font=" . $font .
    " fontsize=16 charref=true");
    
    /* Output the zcaron glyph via a glyph name reference */
    $p->fit_textline("&.zcaron;", $x, $y-=24, "font=" . $font .
        " fontsize=16 charref=false");
    $p->fit_textline("&.zcaron;", $x+160, $y, "font=" . $font .
        " fontsize=16 charref=true");
    
    /* Output the zcaron glyph via a numerical character reference */
    $p->fit_textline("&#x017E;", $x, $y-=24, "font=" . $font .
        " fontsize=16 charref=false");
    $p->fit_textline("&#x017E;", $x+160, $y, "font=" . $font .
        " fontsize=16 charref=true"); 

    /* Finish page */
    $p->end_page_ext("");

    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=character_references.pdf");
    print $buf;

} catch (PDFlibException $e) {
    echo("PDFlib exception occurred:\n".
        "[" . $e->get_errnum() . "] " . $e->get_apiname() .
        ": " . $e->get_errmsg() . "\n");
    exit(1);
} catch (Throwable $e) {
    echo("PHP exception occurred: " . $e->getMessage() . "\n");
    exit(1);
}

$p = 0;

?>

