<?php
/*
 * Embedded video:
 * Create a RichMedia annotation which plays an embedded video.
 *
 * We don't use a Flash-based wrapper which has the following consequences:
 * - The video is played even in Acrobat DC versions which no longer support
 *   Flash.
 * - The video does not work in old Acrobat X/XI versions.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 * Required data: video file

 */
/* This is where the data files are. Adjust as necessary */
$searchpath = dirname(__FILE__,3)."/input";
$outfile = "";
$title = "Embedded video";


/* movie file to be referenced by the link */
$videofile = "mountains.mp4";

try {
    $width = 550; $height = 310;

    $p = new pdflib();

    /* This means we must check return values of load_asset() etc. */
    $p->set_option("errorpolicy=return");
    $p->set_option("stringformat=utf8");

    $p->set_option("searchpath={" . $searchpath . "}");

    if ($p->begin_document($outfile, "compatibility=1.7ext3") == 0)
        throw new Exception("Error: " . $p->get_apiname() . ": "
            . $p->get_errmsg());

    $p->set_info("Creator", "PDFlib Cookbook");
    $p->set_info("Title", $title);

    // Create template for visualizing the video before activation.
    // We display only some text; you could display an image etc.
    
    $templ = $p->begin_template_ext($width, $height, "");
    
        $p->set_graphics_option("fillcolor=red");
        $p->rect(0, 0, $width, $height);
        $p->fill();

        $p->fit_textline("Video", 10, 10,
            "fontname=NotoSerif-Regular encoding=unicode fontsize=14 " .
            "fitmethod=meet fillcolor=black position=center boxsize={" . 
            $width . " " . $height . "}");

    $p->end_template_ext(0, 0);

    
    $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

    // Load video file as RichMedia asset
    $video = $p->load_asset("Video", $videofile, "");
    if ($video == 0)
        throw new Exception("Error: " . $p->get_errmsg());
    
    /* The option list for the RichMedia annotation specifies the
     * template as static placeholder, controls video embedding and
     * defines the the Flash helpers for playing.
     */
 
    $optlist =
    "template={normal=" . $templ . "} " .
    "richmedia={" .
        "assets={ {asset=" . $video . " name=" . $videofile . "} } " .
        "configuration={type=Video instances={ {asset=" . $videofile . "} }}" .
    "}";

    // Create the RichMedia annotation with video
    $p->create_annotation(25, 300, 575, 610, "RichMedia", $optlist);

    $p->end_page_ext("");

    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=movie_annontations.pdf");
    print $buf;

} catch (PDFlibException $e) {
    echo("PDFlib exception occurred:\n".
        "[" . $e->get_errnum() . "] " . $e->get_apiname() .
        ": " . $e->get_errmsg() . "\n");
    exit(1);
} catch (Throwable $e) {
    echo("PHP exception occurred: " . $e->getMessage() . "\n");
    exit(1);
}

$p = 0;

?>
